@@ -10,17 +10,17 @@ use crate::en::{Encode, Encoder, PairEncoder, PairsEncoder, SequenceEncoder};
 use crate::error::Error;
 use crate::internal::size_hint;
 
-impl Encode for String {
+impl<Mode> Encode<Mode> for String {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
-        self.as_str().encode(encoder)
+        Encode::<Mode>::encode(self.as_str(), encoder)
     }
 }
 
-impl<'de> Decode<'de> for String {
+impl<'de, Mode> Decode<'de, Mode> for String {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
@@ -61,37 +61,37 @@ impl<'de> Decode<'de> for String {
     }
 }
 
-impl Encode for Box<str> {
+impl<Mode> Encode<Mode> for Box<str> {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
-        self.as_ref().encode(encoder)
+        Encode::<Mode>::encode(self.as_ref(), encoder)
     }
 }
 
-impl<'de> Decode<'de> for Box<str> {
+impl<'de, Mode> Decode<'de, Mode> for Box<str> {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
         D: Decoder<'de>,
     {
-        Ok(String::decode(decoder)?.into())
+        Ok(<String as Decode<Mode>>::decode(decoder)?.into())
     }
 }
 
-impl Encode for Cow<'_, str> {
+impl<Mode> Encode<Mode> for Cow<'_, str> {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
-        self.as_ref().encode(encoder)
+        Encode::<Mode>::encode(self.as_ref(), encoder)
     }
 }
 
-impl<'de> Decode<'de> for Cow<'de, str> {
+impl<'de, Mode> Decode<'de, Mode> for Cow<'de, str> {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
@@ -139,9 +139,9 @@ macro_rules! sequence {
         $access:ident,
         $factory:expr
     ) => {
-        impl<T $(, $extra)*> Encode for $ty<T $(, $extra)*>
+        impl<T, Mode $(, $extra)*> Encode<Mode> for $ty<T $(, $extra)*>
         where
-            T: Encode,
+            T: Encode<Mode>,
             $($extra: $extra_bound0 $(+ $extra_bound)*),*
         {
             #[inline]
@@ -149,19 +149,19 @@ macro_rules! sequence {
             where
                 E: Encoder,
             {
-                encoder.encode_sequence(self.len(), |mut seq| {
-                    for value in self {
-                        value.encode(seq.next()?)?;
-                    }
+                let mut seq = encoder.encode_sequence(self.len())?;
 
-                    seq.end()
-                })
+                for value in self {
+                    value.encode(seq.next()?)?;
+                }
+
+                seq.end()
             }
         }
 
-        impl<'de, T $(, $extra)*> Decode<'de> for $ty<T $(, $extra)*>
+        impl<'de, Mode, T $(, $extra)*> Decode<'de, Mode> for $ty<T $(, $extra)*>
         where
-            T: Decode<'de> $(+ $trait0 $(+ $trait)*)*,
+            T: Decode<'de, Mode> $(+ $trait0 $(+ $trait)*)*,
             $($extra: $extra_bound0 $(+ $extra_bound)*),*
         {
             #[inline]
@@ -169,15 +169,14 @@ macro_rules! sequence {
             where
                 D: Decoder<'de>,
             {
-                decoder.decode_sequence(|mut $access| {
-                    let mut out = $factory;
+                let mut $access = decoder.decode_sequence()?;
+                let mut out = $factory;
 
-                    while let Some(value) = $access.next()? {
-                        out.$insert(T::decode(value)?);
-                    }
+                while let Some(value) = $access.next()? {
+                    out.$insert(T::decode(value)?);
+                }
 
-                    Ok(out)
-                })
+                Ok(out)
             }
         }
     }
@@ -214,10 +213,10 @@ macro_rules! map {
         $access:ident,
         $with_capacity:expr
     ) => {
-        impl<'de, K, V $(, $extra)*> Encode for $ty<K, V $(, $extra)*>
+        impl<'de, Mode, K, V $(, $extra)*> Encode<Mode> for $ty<K, V $(, $extra)*>
         where
-            K: Encode,
-            V: Encode,
+            K: Encode<Mode>,
+            V: Encode<Mode>,
             $($extra: $extra_bound0 $(+ $extra_bound)*),*
         {
             #[inline]
@@ -225,23 +224,22 @@ macro_rules! map {
             where
                 E: Encoder,
             {
-                encoder.encode_map(self.len(), |mut map| {
-                    for (k, v) in self {
-                        let mut entry = map.next()?;
-                        k.encode(entry.first()?)?;
-                        v.encode(entry.second()?)?;
-                        entry.end()?;
-                    }
+                let mut map = encoder.encode_map(self.len())?;
 
-                    map.end()
-                })
+                for (k, v) in self {
+                    let mut entry = map.next()?;
+                    k.encode(entry.first()?)?;
+                    v.encode(entry.second()?)?;
+                }
+
+                map.end()
             }
         }
 
-        impl<'de, K, V $(, $extra)*> Decode<'de> for $ty<K, V $(, $extra)*>
+        impl<'de, K, V, Mode $(, $extra)*> Decode<'de, Mode> for $ty<K, V $(, $extra)*>
         where
-            K: Decode<'de> $(+ $key_bound0 $(+ $key_bound)*)*,
-            V: Decode<'de>,
+            K: Decode<'de, Mode> $(+ $key_bound0 $(+ $key_bound)*)*,
+            V: Decode<'de, Mode>,
             $($extra: $extra_bound0 $(+ $extra_bound)*),*
         {
             #[inline]
@@ -249,17 +247,16 @@ macro_rules! map {
             where
                 D: Decoder<'de>,
             {
-                decoder.decode_map(|mut $access| {
-                    let mut out = $with_capacity;
+                let mut $access = decoder.decode_map()?;
+                let mut out = $with_capacity;
 
-                    while let Some(mut entry) = $access.next()? {
-                        let key = entry.first().and_then(K::decode)?;
-                        let value = entry.second().and_then(V::decode)?;
-                        out.insert(key, value);
-                    }
+                while let Some(mut entry) = $access.next()? {
+                    let key = entry.first().and_then(K::decode)?;
+                    let value = entry.second().and_then(V::decode)?;
+                    out.insert(key, value);
+                }
 
-                    Ok(out)
-                })
+                Ok(out)
             }
         }
     }
@@ -276,7 +273,7 @@ map!(
     HashMap::with_capacity_and_hasher(size_hint::cautious(map.size_hint()), S::default())
 );
 
-impl Encode for CStr {
+impl<Mode> Encode<Mode> for CStr {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
@@ -286,18 +283,18 @@ impl Encode for CStr {
     }
 }
 
-impl<'de> Decode<'de> for &'de CStr {
+impl<'de, Mode> Decode<'de, Mode> for &'de CStr {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
         D: Decoder<'de>,
     {
-        let bytes = <&[u8]>::decode(decoder)?;
+        let bytes = <&[u8] as Decode<Mode>>::decode(decoder)?;
         CStr::from_bytes_with_nul(bytes).map_err(D::Error::custom)
     }
 }
 
-impl Encode for CString {
+impl<Mode> Encode<Mode> for CString {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
@@ -307,12 +304,12 @@ impl Encode for CString {
     }
 }
 
-impl<'de> Decode<'de> for CString {
+impl<'de, Mode> Decode<'de, Mode> for CString {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
         D: Decoder<'de>,
     {
-        Ok(<&CStr>::decode(decoder)?.to_owned())
+        Ok(<&CStr as Decode<Mode>>::decode(decoder)?.to_owned())
     }
 }
